@@ -10,13 +10,11 @@ NULL
   # BuildDate <- pkgdes$Date
 
   foo <- paste(#"## \n##  magi (Version ", version, ", Build Date: ", BuildDate, ")\n",
-               "##  See vignette https://arxiv.org/abs/2203.06066 for detailed usage instructions, \n",
+               "##  See https://doi.org/10.18637/jss.v109.i04 for a user guide, \n",
+               "##  https://doi.org/10.1073/pnas.2020397118 for details of the MAGI method, \n",
                "##  and https://github.com/wongswk/magi for additional documentation.\n",
                "##  \n",
-               "##  Please cite MAGI method as:\n",
-               "##  Yang, S., Wong, S.W. and Kou, S.C., 2021.\n",
-               "##  Inference of dynamic systems from noisy and sparse data via manifold-constrained Gaussian processes.\n",
-               "##  Proceedings of the National Academy of Sciences, 118(15), e2020397118.\n",
+               '##  To cite the magi software, please see citation("magi")\n',
                sep = "")
   packageStartupMessage(foo)
 }
